// unit slider_bar
//
// extra trackbar with color properties for thumb and track
// and some more properties
//
// version 1.0  2001 Delfi
//
// this component is freeware for non-commercial applications you use it on your own responsibility !
// you may modify this unit only for non-commercial applications
// you may not change the AUTHOR you must leave this comment here !
//
// if you improved this component add your comment after this line

unit slider_bar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls, stdctrls;

type
  Tslider_barOrientation = (slHoriz, slVertical);
  TTickmarks=(tmBottom,tmTop,tmNone);
  Tslider_bar = class(TCustomControl)
  private
    Thumb : TRect;               { Current thumb position             }
    MemDC : HDC;                 { DC for saving area under the thumb }
    Bitmap : HBitmap;            { Bitmap handle for saved area.      }

    capture : boolean;           { Whether it's currently being moved }
    capturePoint : TPoint;       { Position at start of capture.      }
    captureValue : longint;      { Value at start of capture.         }

                                 { Property values...                 }
    fTrackWidth : Integer;
    fNotchLength: byte;
    fTickmarks  :TTickmarks;
    fNumTickmarks  :word;
    fTrackColor : TColor;
    fNotchColor : TColor;
    fOrientation : Tslider_barOrientation;
    fThumbHeight : Integer;
    fSizeTickmarks : byte;
    fThumbColor : TColor;
    fMin : longint;
    fMax : longint;
    fValue : longint;
    fValueChange : TNotifyEvent;
    fCtl3D : boolean;
    firsttime:boolean;
    fhasmousedown:boolean;

    procedure SetTrackWidth (value : Integer);
    procedure SetSizeTickmarks (value : byte);
    procedure SetNotchLength (value : byte);
    procedure SetNumTickmarks (value : word);
    procedure SetTickmarks (value : TTickmarks);
    procedure SetTrackColor (value : TColor);
    procedure SetNotchColor (value : TColor);
    procedure SetOrientation (value : Tslider_barOrientation);
    procedure SetThumbHeight (value : Integer);
    procedure SetThumbColor (value : TColor);
    procedure SetMin (v : longint);
    procedure SetMax (v : longint);
    procedure SetValue (value : longint);
    procedure SetCtl3D (value : boolean);
    procedure CMDialogChar(var Msg:TCMDialogChar);message CM_DIALOGCHAR;
    procedure WMSysChar(var Msg:TWMSysChar);message WM_SYSCHAR;
    procedure CMEnter(var message:TCMGotFocus);message CM_ENTER;
    procedure CMExit(var Message:TCMExit);message CM_EXIT;
    Procedure WMGetDlgCode(var Msg:TWMGetDlgCode);
              message wm_GetDlgCode;
  protected
    procedure Paint; override;
    procedure MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove (Shift: TShiftState; X, Y: Integer); override;
    procedure DrawThumb; virtual;
    procedure DrawTicks; virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyUp(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: char); override;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property HasMouseDown:boolean read fHasMouseDown;
    property iscaptured:boolean read capture;
    property TrackWidth : Integer read fTrackWidth write SetTrackWidth;
    property NotchLength : byte read fNotchLength write SetNotchLength;
    property NumTickmarks : word read fNumTickmarks write SetNumTickmarks;
    property SizeTickmarks : byte read fSizeTickmarks write SetSizeTickmarks;
    property Tickmarks : TTickmarks read fTickmarks write SetTickMarks;
    property NotchColor : TColor read fNotchColor write SetNotchColor;
    property TrackColor : TColor read fTrackColor write SetTrackColor;
    property ThumbHeight : Integer read fThumbHeight write SetThumbHeight;
    property ThumbColor : TColor read fThumbColor write SetThumbColor;
    property Orientation : Tslider_barOrientation read fOrientation write SetOrientation;
    property min : longint read fMin write SetMin;
    property max : longint read fMax write SetMax;
    property Value : longint read fValue write SetValue;
    property Ctl3D : boolean read fCtl3D write SetCtl3D;
    property OnValueChange : TNotifyEvent read fValueChange write fValueChange;

    property Color;
    property Enabled;
    property HelpContext;
    property Hint;
    property ParentShowHint;
    property ShowHint;
    property Tag;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnExit;
    property OnEnter;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property TabOrder;
    property TabStop;
    property OnstartDock;
    property OnstartDrag;
    property Visible;
    property Popupmenu;
    property cursor;
    property dragkind;
    property dragmode;
    property dragcursor;
  end;

procedure Register;

implementation

constructor Tslider_bar.Create (AOwner : TComponent);
begin
  inherited Create (AOwner);
  controlstyle:=controlstyle - [csOpaque];
  fhasmousedown:=false;
  firsttime:=true;
  bitmap:=0;
  tabstop:=true;
  taborder:=1;
  Width := 50;
  Height := 200;
  fTrackWidth := 10;
  fNotchLength:=5;
  fTickmarks := tmNone;
  fNumTickmarks:=10;
  fSizeTickmarks :=0;
  fOrientation := slVertical;
  fNotchColor := clBtnFace;
  fTrackColor := clBtnFace;
  fThumbColor := clBtnFace;
  fMin := 0;
  fMax := 100;
  fValue := 0;
  fThumbHeight := 18;
  fValueChange := Nil;
  fCtl3D := True;
  capture := False;
  thumb.left := -1;
end;

destructor Tslider_bar.Destroy;
begin
  if Bitmap <> 0 then DeleteObject (Bitmap);
  if MemDC <> 0 then DeleteDC (MemDC);
  inherited Destroy
end;

procedure Tslider_bar.SetTrackWidth (value : Integer);
begin
  if fTrackWidth <> value then
  begin
    fTrackWidth := value;
    Invalidate
  end
end;

Procedure Tslider_bar.WMGetDlgCode(var Msg:TWMGetDlgCode);
begin
inherited;
Msg.Result:=dlgc_WantArrows;
end;

procedure Tslider_bar.CMDialogChar(var Msg:TCMDialogChar);
begin
end;

procedure Tslider_bar.WMSysChar(var Msg:TWMSysChar);
begin
end;

procedure Tslider_bar.SetSizeTickmarks (value : byte);
begin                  
  if fSizeTickmarks <> value then
  begin
    fSizeTickmarks := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetNotchLength (value : byte);
begin
  if fNotchLength <> value then
  begin
    fNotchLength := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetNumTickmarks (value : word);
begin
  if fNumTickmarks <> value then
  begin
    fNumTickmarks := value;
    Invalidate
  end;
    repaint;
end;

procedure Tslider_bar.SetTickmarks (value : TTickmarks);
begin
  if fTickmarks <> value then
  begin
    fTickmarks := value;
    Invalidate
  end;
  repaint;
end;

procedure Tslider_bar.SetOrientation (value : Tslider_barOrientation);
begin
  if value <> fOrientation then
  begin
{    if csDesigning in ComponentState then
    begin
      hw:=width;
      width:=height;
      height:=hw;
    end;}
    fOrientation := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetTrackColor (value : TColor);
begin
  if value <> fTrackColor then
  begin
    fTrackColor := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetNotchColor (value : TColor);
begin
  if value <> fNotchColor then
  begin
    fNotchColor := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetThumbHeight (value : Integer);
begin
  if value <> fThumbHeight then
  begin
    fThumbHeight := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetThumbColor (value : TColor);
begin
  if value <> fThumbColor then
  begin
    fThumbColor := value;
    Invalidate
  end
end;

procedure Tslider_bar.SetMin (v : longint);
begin
try
  if v <> fMin then
  begin
    fMin := V;
    if Value < fMin then Value := fMin;
    Invalidate
  end
  except end;
end;

procedure Tslider_bar.SetMax (v : longint);
begin
  if v <> fMax then
  begin
    fMax := V;
    if Value > fMax then Value := fMax;
    Invalidate
  end
end;

procedure Tslider_bar.SetValue (value : longint);
begin
repaint;
//if Value = 0 then repaint;
  if value < min then value := min else if value > max
  then value := max;
  if value <> fValue then
  begin
    fValue := Value;
    if Assigned (fValueChange) then OnValueChange (self);
    invalidate;
  end;
paint;
invalidate;
end;

procedure Tslider_bar.SetCtl3D (value : boolean);
begin
  if value <> fCtl3D then
  begin
    fCtl3D := value;
    Invalidate
  end
end;

(*-------------------------------------------------------------------------*
 | procedure Tslider_bar.Paint                                                 |
 |                                                                         |
 | Paint the entire slider_bar control.                                        |
 *-------------------------------------------------------------------------*)
procedure Tslider_bar.Paint;
var Rect : TRect;
begin
canvas.Brush.Style:=bsSolid;
canvas.Brush.Color := fTrackColor;
canvas.FillRect(Rect);
  with Canvas do
  begin                             { Create memory DC for save bitmap }
    if MemDC = 0 then MemDC := CreateCompatibleDC (Canvas.Handle);

                                    { Create thumb & bitmap for vertical slider_bar }
    if fOrientation = slVertical then
    begin
      if Bitmap = 0 then
      begin
        Bitmap := CreateCompatibleBitmap (Canvas.Handle, Width, ThumbHeight);
      end;

      if fTickmarks=tmBottom then
      Rect.left := (width - TrackWidth) div 2 - fSizeTickmarks div 2 else
      if fTickmarks=tmTop then
      Rect.left := (width - TrackWidth) div 2 + fSizeTickmarks div 2 else
      Rect.left := (width - TrackWidth) div 2;
      Rect.top := 0;
      Rect.bottom := Height;
{      Rect.left := (Width - TrackWidth) div 2;}
      Rect.Right := Rect.Left + TrackWidth
    end
    else
    begin
                                    { Create thumb & bitmap for horiz slider_bar }
      if Bitmap = 0 then
        Bitmap := CreateCompatibleBitmap (Canvas.Handle, ThumbHeight, Height);

      if fTickmarks=tmBottom then
      Rect.top := (Height - TrackWidth) div 2 - fSizeTickmarks div 2 else
      if fTickmarks=tmTop then
      Rect.top := (Height - TrackWidth) div 2 + fSizeTickmarks div 2 else
      Rect.top := (Height - TrackWidth) div 2;
      Rect.bottom := Rect.Top + TrackWidth - fSizeTickmarks div 2;
      Rect.left := 0;
      Rect.Right := Width
    end;
                                     { Draw track.                            }
    Brush.Color := TrackColor;
    if Ctl3D then
    begin
      Pen.Color := clBtnHighlight;
      with Rect do
      begin
        Rectangle (left+3, top+1, right-2, bottom-1);
        Pen.Color := clBtnShadow;
        MoveTo (left+2, top+1);
        LineTo (right-1, top+1);
        MoveTo (left+3, top+1);
        LineTo (left+3, bottom-1);

        Pen.Color := clblack;
        MoveTo (Left+2, top);
        LineTo (Right-2, top);
        LineTo (Right-2, bottom-1);
        LineTo (left+2, bottom-1);
        LineTo (left+2, top);
      end
    end
    else FillRect (Rect);            { Not Ctl3D - do a simple rectangle      }

    if fTickmarks<>tmNone then
    DrawTicks;                       { Now draw the ticks                     }

    DrawThumb;                       { Now draw the thumb.                    }

  end;
//    firsttime:=false;
end;

procedure Tslider_bar.DrawTicks;
var i:integer;
    he:word;
begin
  with Canvas do
  begin
    pen.color:=clblack;
    if fOrientation = slVertical then
    begin
      if fTickmarks=tmBottom then
      he:=width else
      if fTickmarks=tmTop then
      he:=fSizeTickmarks-2 else he:=fSizeTickmarks div 2;
      for i:=0 to fNumTickMarks do
      begin
        moveto(he,fThumbHeight div 2+round(((height-fThumbHeight) / fNumTickMarks)*i)
          );
        lineto(he-fSizeTickmarks,fThumbHeight div 2+round(((height-fThumbHeight)
          / fNumTickMarks)*i));
      end;
    end else
    begin
      if fTickmarks=tmBottom then
      he:=height else
      if fTickmarks=tmTop then
      he:=fSizeTickmarks-2 else he:=fSizeTickmarks div 2;
      for i:=0 to fNumTickMarks do
      begin
        moveto(fThumbHeight div 2+round(((width-fThumbHeight) / fNumTickMarks)*i)
          ,he);
        lineto(fThumbHeight div 2+round(((width-fThumbHeight) / fNumTickMarks)*i)
           ,he-fSizeTickmarks);
      end;
    end;
  end;
end;

(*-------------------------------------------------------------------------*
 | procedure Tslider_bar.DrawThumb                                             |
 |                                                                         |
 | Draw the thumb at the correct position for the current value.           |
 *-------------------------------------------------------------------------*)
procedure Tslider_bar.DrawThumb;
var
  basePos : Integer;
  oldBmp : HBitmap;
  oldThumb : TRect;
begin
try
  if csLoading in ComponentState then Exit;

  oldBmp := SelectObject (MemDC, Bitmap);

  { Set thumb color & border color }
  if Enabled then Canvas.Brush.Color := ThumbColor else Canvas.Brush.Color := clGray;
  if Ctl3D then Canvas.Pen.Color := clBtnHighlight else Canvas.Pen.Color := clBTNFACE;

  { Save current thum rectangle }
  oldThumb := Thumb;

  if Orientation = slVertical then
  begin

  { Calculate new thumb rectangle }
    basePos := LongInt (Height - ThumbHeight) * LongInt (Value - min) div (max - min);
{    Thumb.left := 0;
    Thumb.right := Width;}
    Thumb.Bottom := Height - BasePos;
    Thumb.top := Thumb.Bottom - ThumbHeight;
    if fTickmarks=tmBottom then
    Thumb.left := 1 else
    if fTickmarks=tmTop then
    thumb.left:=fSizeTickmarks else
    thumb.left:=fSizeTickmarks div 2;
    Thumb.right := thumb.left+width - fSizeTickmarks-1;

  { Rub out old thumb}
  if (oldThumb.left <> -1) then with oldThumb do
      BitBlt (Canvas.Handle, Left, Top, right-left, ThumbHeight, MemDC, 0, 0, SRCCOPY);
//      Painttrack;

  { Save what's underneath         }
   if (oldThumb.left = -1) then
    with Thumb do
      BitBlt (MemDC, 0, 0, right-left, ThumbHeight, Canvas.Handle, Left, {Top}1, SRCCOPY);
  end
  else
  begin
  { Calculate new thumb rectangle for horiz slider_bar}
    basePos := LongInt (Width - ThumbHeight) * LongInt(Value - min) div (max - min);
    Thumb.left := basePos;
    Thumb.Right := Thumb.left + ThumbHeight;
    if fTickmarks=tmBottom then
    Thumb.Top := 1 else
    if fTickmarks=tmTop then
    thumb.top:=fSizeTickmarks else
    thumb.top:=fSizeTickmarks div 2;
    Thumb.Bottom := thumb.top+Height - fSizeTickmarks-1;

  { Rub out old thumb             }
    if (oldThumb.left <> -1) then with oldThumb do
      BitBlt (Canvas.Handle, Left, Top, ThumbHeight, bottom-top, MemDC, 0, 0, SRCCOPY);
//      Painttrack;

  { Save what's underneath         }
   if (oldThumb.left=-1) then
   begin
    with Thumb do
      BitBlt (MemDC, 0, 0, ThumbHeight, bottom-top, Canvas.Handle, {Left+1}1, Top, SRCCOPY);
   end;
  end;

  { Draw the thumb                 }
  with Canvas do
  begin
    with Thumb do if Ctl3D then
    begin
      Rectangle (left+1, top+1, right-1, bottom-1);
      Pen.Color := clBtnShadow;
      MoveTo (Left+2, Bottom - 2);
      LineTo (Right-2, Bottom - 2);
      LineTo (Right-2, Top+1);

      Pen.Color := clblack;
      MoveTo (Left, top);
      LineTo (Right-1, top);
      LineTo (Right-1, bottom-1);
      LineTo (left, bottom-1);
      LineTo (left, top);
    end
    else

      Rectangle (left, top, right, bottom);

    {draw notch}

  Pen.Color := fNotchColor;
  with thumb do
  if Orientation = slVertical then
  begin
    if fTickmarks=tmBottom then
    begin
//      MoveTo (right-4, top+(bottom-top) div 2);
//      LineTo (right-4-fNotchLength,top+(bottom-top) div 2);
    end else
    if fTickmarks=tmTop then
    begin
//      MoveTo (left+3,top+(bottom-top) div 2);
//      LineTo (left+3+fNotchLength,top+(bottom-top) div 2);
    end else {do nothing}
  end else
  begin
    if fTickmarks=tmBottom then
    begin
//      MoveTo (left+(right-Left) div 2, Bottom - 4);
//      LineTo (left+(right-Left) div 2, Bottom - 4-fNotchLength);
    end else
    if fTickmarks=tmTop then
    begin
//      MoveTo (left+(right-Left) div 2, top + 3);
//      LineTo (left+(right-Left) div 2, top + 3+fNotchLength);
    end else {do nothing}
  end;
  Pen.Color := clBtnShadow;
  end;
  SelectObject (MemDC, OldBmp);
  except end;
end;

(*-------------------------------------------------------------------------*
 | procedure Tslider_bar.MouseDown (button, Shift, X, Y)                       |
 |                                                                         |
 | Respond to 'MouseDown' events.  Record the position of press for use in |
 | the MouseMove event handler.                                            |
 |                                                                         |
 | Parameters:                                                             |
 |   button : TMouseButton         Set of mouse buttons currently pressed. |
 |   Shift  : TShiftState          Set of shift, ctrl, alt keys            |
 |   X, Y   : Integer              Mouse position relative to top of slider_bar|
 *-------------------------------------------------------------------------*)
procedure Tslider_bar.MouseDown (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var hr:trect;
begin
  inherited MouseDown (Button, Shift, X, Y);
  if not visible then exit;
  setfocus;
  if (Button = mbLeft) and PtInRect (Thumb, Point (X, Y)) then
  begin
    fhasmousedown:=true;
    capture := True;
    capturePoint := Point (X, Y);
    captureValue := value;
  end else
    with hr do
    begin
      if forientation=slHoriz then
      begin
        right:=thumb.left;
        bottom:=height;
        left:=0;
        top:=0;
      end else
      begin
        top:=thumb.bottom;
        right:=width;
        bottom:=height;
        left:=0;
      end;
      if (Button = mbLeft) and PtInRect (hr, Point (X, Y)) then
      begin
        value:=value-1;
        exit;
      end;
      if forientation=slHoriz then
      begin
        left:=thumb.right;
        bottom:=height;
        right:=width;
        top:=0;
      end else
      begin
        bottom:=thumb.top;
        right:=width;
        top:=0;
        left:=0;
      end;
      if (Button = mbLeft) and PtInRect (hr, Point (X, Y)) then
      begin
        value:=value+1;
      end;
    end;
  begin
  end;
end;

(*-------------------------------------------------------------------------*
 | procedure Tslider_bar.MouseUp (button, Shift, X, Y)                         |
 |                                                                         |
 | Respond to 'MouseUp' events                                             |
 |                                                                         |
 | Parameters:                                                             |
 |   button : TMouseButton         Set of mouse buttons currently pressed. |
 |   Shift  : TShiftState          Set of shift, ctrl, alt keys            |
 |   X, Y   : Integer              Mouse position relative to top of slider_bar|
 *-------------------------------------------------------------------------*)
procedure Tslider_bar.MouseUp (Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp (Button, Shift, X, Y);
  if (Button = mbLeft) then
  begin
    capture := False;
    fhasmousedown:=false;
  end;
value:=value;
end;

(*-------------------------------------------------------------------------*
 | procedure Tslider_bar.MouseMove (Shift, X, Y)                               |
 |                                                                         |
 | Respond to 'MouseMove' events                                           |
 |                                                                         |
 | Parameters:                                                             |
 |   Shift  : TShiftState          Set of shift, ctrl, alt keys            |
 |   X, Y   : Integer              Mouse position relative to top of slider_bar|
 *-------------------------------------------------------------------------*)
procedure Tslider_bar.MouseMove (Shift: TShiftState; X, Y: Integer);
begin
inherited MouseMove (shift, X, Y);
if capture then
if Orientation = slVertical then
value := captureValue {+ min} + LongInt (max - min) *LongInt(capturePoint.Y - Y) div (Height - ThumbHeight)
else
value := captureValue{ + min} + LongInt (max - min) * LongInt(X - capturePoint.X) div (Width - ThumbHeight);
//value:=value;
end;

procedure Tslider_bar.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited keydown(key,shift);
  fhasmousedown:=true;
if fOrientation = slVertical then begin
Case Key Of
     vk_End:value:=Fmin;
     vk_Home:value:=Fmax;
     vk_Left,vk_down: value:=value-1;
     vk_Right,vk_up:value:=value+1;
     VK_PRIOR:value:=value-1;
     VK_NEXT:value:=value+1;
    end;
end;
if fOrientation = slHoriz then begin
Case Key Of
     vk_End:value:=Fmax;
     vk_Home:value:=Fmin;
     vk_Left,vk_down: value:=value-1;
     vk_Right,vk_up:value:=value+1;
     VK_PRIOR:value:=value-1;
     VK_NEXT:value:=value+1;
    end;
end;
paint;
end;

procedure Tslider_bar.KeyUp(var Key: Word; Shift: TShiftState);
begin
  inherited keyup(key,shift);
  fhasmousedown:=false;
end;

procedure Tslider_bar.KeyPress(var Key: char);
begin
  case key of
    '-':value:=value-1;
    '+':value:=value+1;
  else
    inherited keypress(key);
  end;
end;

procedure Tslider_bar.CMEnter(var message:TCMGotFocus);
begin
  invalidate;
  inherited;
end;
procedure Tslider_bar.CMExit(var Message:TCMExit);
begin
  invalidate;
  inherited;
end;

procedure Register;
begin
  RegisterComponents('Delfi', [Tslider_bar]);
end;

end.
